package com.mamezou.employee.web;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.servlet.ServletException;

import com.mamezou.employee.business.Employee;
import com.mamezou.employee.business.EmployeeManager;
import com.mamezou.employee.validation.EmployeeValidator;

import uk.ltd.getahead.dwr.WebContext;
import uk.ltd.getahead.dwr.WebContextFactory;

/**
 * Ajaxy[W痘pt@T[hNX
 * @author MBeans
 * @version 1.0
 */
public class AjaxFacade {

    // ]ƈǗ̃rWlXIuWFNg
    private EmployeeManager employeeManager;

    // ]ƈǗ̃rWlXIuWFNgp̃Zb^[CWFNV
    public void setEmployeeManager(EmployeeManager employeeManager) {
        this.employeeManager = employeeManager;
    }

    /**
     * w肳ꂽꂩn܂閼Ȍ]ƈꗗ擾B
     * 
     * @param searchWord 
     * @return w肳ꂽꂩn܂閼Ȍ]ƈꗗ
     */
    public List<Employee> filterEmployees(String searchWord) {
        List<Employee> allEmployees = employeeManager.findAll();
        if (searchWord == null) {
            return allEmployees;
        }
        searchWord = searchWord.trim();
        if (searchWord.length() == 0) {
            return allEmployees;
        }

        List<Employee> employees = new ArrayList<Employee>();
        for (Employee employee : allEmployees) {
            if (!employee.getName().startsWith(searchWord)) {
                continue;
            }
            employees.add(employee);
        }
        return employees;
    }

    // ]ƈꗗŏIXV̍~Ƀ\[gRp[^
    private static final Comparator<Employee> LAST_UPDATE_DATE_COMPARATOR
            = new Comparator<Employee>() {
                public int compare(Employee emp1, Employee emp2) {
                    return emp2.getLastUpdateDate()
                            .compareTo(emp1.getLastUpdateDate());
                }
            };

    /**
     * ]ƈ̍XV\HTML擾B
     * 
     * @param historySize 擾闚
     * @return ]ƈ̍XV\HTML
     * @throws IllegalArgumentException 𐔂1̏ꍇ
     */
    public String getEmployeeUpdateHistoryHTML(int historySize) {
        if (historySize < 1) {
            throw new IllegalArgumentException(
                    "historySize must specify 1 or more.");
        }
        List<Employee> employees = employeeManager.findAll();
        if (1 < employees.size()) {
            Collections.sort(employees, LAST_UPDATE_DATE_COMPARATOR);
        }
        if (historySize < employees.size()) {
            employees = employees.subList(0, historySize);
        }
        WebContext webContext = WebContextFactory.get();
        webContext.getHttpServletRequest().setAttribute("employees", employees);
        try {
            return webContext.forwardToString(
                    "/WEB-INF/jsp/portion/employeeUpdateHistory.jsp");
        } catch (ServletException e) {
            e.printStackTrace();
            return "";
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * ]ƈǉB
     * 
     * @param emp ǉ]ƈ
     */
    public void addEmployee(Employee emp) {
        employeeManager.addEmployee(emp);
    }

    /**
     * ]ƈ폜B
     * 
     * @param emp 폜]ƈ
     */
    public void removeEmployee(Employee emp) {
        employeeManager.removeEmployee(emp);
    }

    /**
     * w肳ꂽ]ƈ̖OL𔻒肷B
     * 
     * @param employeeName ]ƈ̖O
     * @return Lȏꍇ͋󃊃XgBȏꍇ̓G[bZ[W̃Xg
     */
    public List<String> isValidEmployeeName(String employeeName) {
        List<String> errors = new ArrayList<String>();
        if (!EmployeeValidator.isString(employeeName)) {
            errors.add("O͂ĉB");
        }
        return errors;
    }

    /**
     * w肳ꂽ]ƈ̏ZL𔻒肷B
     * 
     * @param employeeName ]ƈ̏Z
     * @return Lȏꍇ͋󃊃XgBȏꍇ̓G[bZ[W̃Xg
     */
    public List<String> isValidEmployeeAddress(String employeeAddress) {
        List<String> errors = new ArrayList<String>();
        if (!EmployeeValidator.isString(employeeAddress)) {
            errors.add("Z͂ĉB");
        }
        return errors;
    }
}
