/*
 * HsqldbServer.java
 */
package com.mamezou.utils.hsql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import org.hsqldb.Server;

/**
 * @author MBeans
 * @version 1.0
 */
public class HsqldbServer {

    private static final String DEFAULT_PORT = "9001";
    private static final boolean DEFAULT_NO_SYSTEM_EXIT = true;

    private String database;
    private String port;
    private String noSystemExit;

    public HsqldbServer(String database) {
        this(database, DEFAULT_PORT, DEFAULT_NO_SYSTEM_EXIT);
    }

    public HsqldbServer(String database, String port) {
        this(database, port, DEFAULT_NO_SYSTEM_EXIT);
    }

    public HsqldbServer(String database, String port, boolean noSystemExit) {
        if (database == null) {
            throw new IllegalArgumentException("database is null.");
        }
        if (port == null) {
            throw new IllegalArgumentException("port is null.");
        }
        this.database = database;
        this.port = port;
        this.noSystemExit = Boolean.toString(noSystemExit);
    }

    public void start() {
        Thread server = new Thread() {
                            public void run() {
                                Server.main(new String[] {
                                        "-database", database,
                                        "-port", port,
                                        "-no_system_exit", noSystemExit});
                            }
                        };
        server.start();
    }

    public void stop() {
        String driver = "org.hsqldb.jdbcDriver";
        String port = "9001";
        String url = "jdbc:hsqldb:hsql://localhost" + ":" + port;
        String user = "sa";
        String password = "";
        Connection connection = null;
        Statement statement = null;
        try {
            Class.forName(driver);
            connection = DriverManager.getConnection(url, user, password);
            statement = connection.createStatement();
            statement.execute("SHUTDOWN");
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            close(connection, statement);
        }
    }

    private void close(Connection connection, Statement statement) {
        try {
            if (statement != null) {
                statement.close();
                connection = null;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }
}
